import 'dart:async';
import 'dart:convert';
import 'dart:core';
import 'package:flutter/services.dart';

class PaygilantPlugin {
  static const MethodChannel _channel = const MethodChannel('paygilant_plugin');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// Initializes Paygilant service. This function must be invoked before any other Paygilant function
  ///-  param [serverUrl] - url address of Paygilant server
  /// - param [userId] - userId or username (can be omitted if unknown)
  static Future<void> init(String serverUrl, String userId) async {
    return _channel.invokeMethod('init', <String, dynamic>{
      'serverUrl': serverUrl,
      'userId': userId,
    });
  }

  /// Initialized device Id
  static Future<void> initializeDeviceId() async {
    return _channel.invokeMethod('initializeDeviceId');
  }

  /// logout
  static Future<void> logout() async {
    return _channel.invokeMethod('logout');
  }

  static Future<String> get getDeviceId async {
    final String getDeviceId = await _channel.invokeMethod('getDeviceId');
    return getDeviceId;
  }

  static Future<String> get getSessionId async {
    final String getSessionId = await _channel.invokeMethod('getSessionId');
    return getSessionId;
  }

  ///Sends information that app is paused (standby, user switched to other app).
  static Future<void> pauseListenToSensors() async {
    return _channel.invokeMethod('pauseListenToSensors');
  }

  ///Start screen listener - stating bio monitoring to a given screen.
  ///param {ScreenListenerType} [ScreenListenerType] - type of screen listener
  ///param {int} [screenId] - Screen id number
  static Future<void> startNewScreenListener(
      ScreenListenerType screenListenerType, int screenId) async {
    return _channel.invokeMethod('startNewScreenListener', <String, dynamic>{
      'screenListenerType': enumToString(screenListenerType),
      'screenId': screenId,
    });
  }

  ///Sends info to Paygilant server that user has arrived to checkpoint
  /// -param {Checkpoint} [checkPoint]
  static Future<void> arriveToCheckPoint(CheckPoint checkpoint) async {
    var convertStr = json.encode(checkpoint.getJson());
    return _channel.invokeMethod('arriveToCheckPoint', <String, dynamic>{
      'checkpoint': convertStr,
    });
  }

  /// Gets risk for checkpoint
  /// param {Checkpoint} [checkPoint]
  /// returns {Map} [risk] score
  static Future<Map<String, dynamic>> getRiskForCheckPoint(
      CheckPoint checkpoint) async {
    var jsonData = checkpoint.getJson();
    var convertStr = json.encode(jsonData);

    final String risk =
        await _channel.invokeMethod('getRiskForCheckPoint', <String, dynamic>{
      'checkpoint': convertStr,
    });
    return json.decode(risk);
  }

  /// Updates checkpoint
  /// -param {CheckpointType} [checkPointType] - type of checkpoint
  /// -param {String} [requestId] - request id
  /// -param {CheckPointStatus} [checkPointStatus] - decision as made by the customer related to the checkpoint
  /// {String} [transactionID] - transaction id
  static Future<void> updateCheckPointStatus(
      CheckPointType checkPointType,
      String requestID,
      CheckPointStatus checkPointStatus,
      String transactionID) async {
    return _channel.invokeMethod('updateCheckPointStatus', <String, dynamic>{
      'checkPointType': enumToString(checkPointType),
      'requestID': requestID,
      'checkPointStatus': enumToString(checkPointStatus),
      'transactionID': transactionID,
    });
  }

  ///Request Permissions Result - list / single
  /// -param {List<String>} [result] - result
  /// Android: "camera","contacts", "location","locationAlways","locationWhenInUse","phone", "storage", "calendar"
  /// iOS: "contacts","locationWhenInUse","locationAlways","location","photos","camera","reminders","calendar","appTrackingTransparency"
  static Future<void> onRequestPermissionsResult(List<String> result) async {
    return _channel
        .invokeMethod('onRequestPermissionsResult', <String, dynamic>{
      'permissions': result,
    });
  }

  static String enumToString(Object o) => o.toString().split('.').last;

  static void registerWith(registrar) {}
}

/// Generic checkpoint class, all used checkpoint objects inherit that class
/// -param {CheckpointType} [checkPointType] - type of checkpoint
/// -param {Map<String,dynamic>} [additionalData]
abstract class CheckPoint extends ParseObjectToJson {
  CheckPointType checkPointType;
  Map<String, dynamic> additionalData = Map<String, dynamic>();
  CheckPoint({this.checkPointType});
  Map<String, dynamic> getJson();
}

/// Launch checkpoint - can be used in the launch of the app.
/// -param {Map<String,dynamic>} [additionalData]
class Launch extends CheckPoint {
  Launch() : super(checkPointType: CheckPointType.LAUNCH);
  @override
  Map<String, dynamic> getJson() {
    var json = Map<String, dynamic>();
    json["checkpoint_type"] = PaygilantPlugin.enumToString(this.checkPointType);
    json["additionalData"] =
        this.additionalData == null ? null : this.additionalData;
    return json;
  }
}

/// General checkpoint
/// -param {Map<String,dynamic>} [additionalData]
class General extends CheckPoint {
  General() : super(checkPointType: CheckPointType.GENERAL);
  @override
  Map<String, dynamic> getJson() {
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// PasswordChange checkpoint - to be used during PasswordChange attempt
/// -param {User} [user] - username or user id
/// -param {Map<String,dynamic>} [additionalData]
class PasswordChange extends CheckPoint {
  User user;
  PasswordChange({this.user})
      : super(checkPointType: CheckPointType.PASSWORD_CHANGE);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// ViewCardDetails checkpoint - to be used during ViewCardDetails attempt
/// -param {User} [user] - user Obj
/// -param {CreditCardDetail} [creditCardDetail] - CreditCardDetail Obj
/// -param {Map<String,dynamic>} [additionalData]
class ViewCardDetails extends CheckPoint {
  User user;
  CreditCardDetail creditCardDetail;
  ViewCardDetails({this.user, this.creditCardDetail})
      : super(checkPointType: CheckPointType.VIEW_CARD_DETAILS);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseJson(this.creditCardDetail, "creditCardDetail");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// UpdateUserInfo checkpoint - to be used during UpdateUserInfo attempt
/// -param {User} [user] - username or user id
/// -param {Map<String,dynamic>} [additionalData]
class UpdateUserInfo extends CheckPoint {
  User user;
  UpdateUserInfo({this.user})
      : super(checkPointType: CheckPointType.UPDATE_USER_INFO);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// Login checkpoint - to be used during login attempt
/// -param {User} [user] - username or user id
/// -param {Map<String,dynamic>} [additionalData]
class Login extends CheckPoint {
  User user;
  Login({this.user}) : super(checkPointType: CheckPointType.LOGIN);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// Register checkpoint - to be used after registration of new user.
/// -param {User} [user] - username or user id
/// -param {Map<String,dynamic>} [additionalData]
class Registration extends CheckPoint {
  User user;
  Registration({this.user}) : super(checkPointType: CheckPointType.REGISTER);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

/// Add Payment Method checkpoint - use when user add / change his payment method, e.g add credit card details or bank account details
/// -param {User} [user]
/// -param {Payment} [payment]
/// -param {Address} [billingAddress]
/// -param {Map<String,dynamic>} [additionalData]
class AddPaymentMethod extends CheckPoint {
  User user;
  Payment payment;
  Address billingAddress;

  AddPaymentMethod({this.user, this.payment, this.billingAddress})
      : super(checkPointType: CheckPointType.ADD_PAYMENT_METHOD);
  @override
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseJson(this.payment, "payment");
    parseJson(this.billingAddress, "billingAddress");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    return super.json;
  }
}

///Transaction checkpoint to be used on a transaction attempt
/// -param {User} [user]
/// -param {int} [amount]
/// -param {CurrencyCode} [curType] - type of currency
/// -param {TransactionType} [transactionType]
/// -param {String} [destinationId] - end user that transaction is going to
/// -param {Payment} [payment]
/// -param {Address} [billingAddress]
/// -param {Address} [shippingAddress]
/// -param {AuthorizationResponse} [authorizationResponse]
/// -param {Map<String,dynamic>} [additionalData]
/// -param {MerchantDetails} [merchantDetails]
class Transaction extends CheckPoint {
  TransactionType transactionType;
  CurrencyCode curType;
  String destinationId;
  double amount;
  User user;
  Address billingAddress;
  Address shippingAddress;
  Payment payment;
  AuthorizationResponse authorizationResponse;
  MerchantDetails merchantDetails;
  Transaction(
      {this.transactionType,
      this.curType,
      this.destinationId,
      this.amount,
      this.user,
      this.billingAddress,
      this.shippingAddress,
      this.payment,
      this.authorizationResponse,
      this.merchantDetails})
      : super(checkPointType: CheckPointType.TRANSACTION);
  Map<String, dynamic> getJson() {
    parseJson(this.user, "user");
    parseEnum(this.transactionType, "transactionType");
    parseEnum(this.curType, "curType");
    parseObject(this.destinationId, "destinationId");
    parseObject(this.amount, "amount");
    parseJson(this.billingAddress, "billingAddress");
    parseJson(this.shippingAddress, "shippingAddress");
    parseJson(this.payment, "payment");
    parseJson(this.authorizationResponse, "authorizationResponse");
    parseEnum(this.checkPointType, "checkpoint_type");
    parseObject(this.additionalData, "additionalData");
    parseJson(this.merchantDetails, "merchant_details");
    return super.json;
  }
}

/// User object, holds all user related data, such as app user-id, email and address.
/// Each checkpoint object contains a user data object.
/// -param {String} [userId]
/// -param {String} [email]
/// -param {VerificationType} [isEmailVerified]
/// -param {String} [phoneUser]
/// -param {VerificationType} [isPhoneUserVerified]
/// -param {Address} [address]
/// -param {VerificationType} [isKYCVerified]
class User extends ParseObjectToJson {
  String userId;
  String email;
  VerificationType isEmailVerified = VerificationType.UNKNOWN;
  String phoneUser;
  VerificationType isPhoneVerified = VerificationType.UNKNOWN;
  Address address;
  VerificationType isKYCVerified = VerificationType.UNKNOWN;
  User(
      {this.userId,
      this.email,
      this.isEmailVerified,
      this.phoneUser,
      this.isPhoneVerified,
      this.address,
      this.isKYCVerified});
  Map<String, dynamic> getJson() {
    parseObject(this.userId, "user_id");
    parseObject(this.email, "email");
    parseEnum(this.isEmailVerified, "isEmailVerified");
    parseObject(this.phoneUser, "phoneUser");
    parseEnum(this.isPhoneVerified, "isPhoneVerified");
    parseJson(this.address, "user_address");
    parseEnum(this.isKYCVerified, "isKYCVerified");
    return super.json;
  }
}

// Merchant Details information
/// -param {String} [merchant_name]
/// -param {String} [merchant_code]
/// -param {Address} [merchant_address]
class MerchantDetails extends ParseObjectToJson {
  String merchantName;
  String merchantCode;
  Address merchantAddress;
  MerchantDetails({this.merchantName, this.merchantCode, this.merchantAddress});
  Map<String, dynamic> getJson() {
    // var json = Map<String, dynamic>();
    parseObject(this.merchantName, "merchant_name");
    parseObject(this.merchantCode, "merchant_code");
    parseJson(this.merchantAddress, "merchant_address");
    return super.json;
  }
}

/// Address object, holds relevent data for a given address.
/// Address object can be related to shipping, billing or user address.
/// -param {String} [firstName]
/// -param {String} [lastName]
/// -aram {String} [addressLine1]
/// -param {String} [addressLine2]
/// -param {String} [city]
/// -param {String} [state]
/// -param {String} [country]
/// -param {String} [postalCode]
/// -param {String} [phoneNumber]
class Address extends ParseObjectToJson {
  String firstName;
  String lastName;
  String addressLine1;
  String addressLine2;
  String city;
  String state;
  String country;
  String postalCode;
  String phoneNumber;

  Address(
      {this.firstName,
      this.lastName,
      this.addressLine1,
      this.addressLine2,
      this.city,
      this.state,
      this.country,
      this.postalCode,
      this.phoneNumber});
  Map<String, dynamic> getJson() {
    parseObject(this.firstName, "firstName");
    parseObject(this.lastName, "lastName");
    parseObject(this.addressLine1, "addressLine1");
    parseObject(this.addressLine2, "addressLine2");
    parseObject(this.city, "city");
    parseObject(this.state, "state");
    parseObject(this.country, "country");
    parseObject(this.postalCode, "postalCode");
    parseObject(this.phoneNumber, "phoneNumber");
    return super.json;
  }
}

class ParseObjectToJson {
  var json = Map<String, dynamic>();
  jsonStringParse() {}
  void parseObject(dynamic data, String key) {
    if (data != null) {
      json[key] = data;
    }
  }

  void parseJson(dynamic data, String key) {
    if (data != null) {
      json[key] = data.getJson();
    }
  }

  void parseEnum(dynamic data, String key) {
    if (data != null) {
      json[key] = PaygilantPlugin.enumToString(data);
    }
  }
}

///Payment options information, holds information regarding the payment option, if its bank account or credit, method ext.
/// -param {PaymentMethodType} [paymentMethod]
/// -param {String} [processor]
/// -param {String} [fullNameOnCard]
/// -param {CreditCardDetail} [creditCardDetail]
/// -param {BankAccountDetails} [bankAccountDetails]
class Payment extends ParseObjectToJson {
  PaymentMethodType paymentMethodType;
  String processor;
  String fullNameOnCard;
  String paymentMethodId;
  CreditCardDetail creditCardDetail;
  BankAccountDetails bankAccountDetails;
  Payment(
      {this.paymentMethodType,
      this.processor,
      this.fullNameOnCard,
      this.paymentMethodId,
      this.creditCardDetail,
      this.bankAccountDetails});
  Map<String, dynamic> getJson() {
    parseEnum(this.paymentMethodType, "paymentMethodType");
    parseObject(processor, "processor");
    parseObject(paymentMethodId, "paymentMethodId");
    parseJson(creditCardDetail, "creditCardDetail");
    parseJson(bankAccountDetails, "bankAccountDetails");
    return super.json;
  }
}

/// Credit card details
/// -param {String} [bin]
/// -param {String} [lastFourDigit]
/// -param {int} [yearExpiryDate]
/// -param {int} [monthExpiryDate]
/// -param {String} [card_token]
/// -param {String} [cardId]
class CreditCardDetail extends ParseObjectToJson {
  String cardToken;
  String cardId;
  String bin;
  String lastFourDigit;
  int yearExpiryDate;
  int monthExpiryDate;
  CreditCardDetail(
      {this.cardToken,
      this.cardId,
      this.bin,
      this.lastFourDigit,
      this.yearExpiryDate,
      this.monthExpiryDate});
  Map<String, dynamic> getJson() {
    parseObject(this.cardToken, "cardToken");
    parseObject(this.cardId, "cardId");
    parseObject(this.bin, "bin");
    parseObject(this.lastFourDigit, "lastFourDigit");
    parseObject(this.yearExpiryDate, "yearExpiryDate");
    parseObject(this.monthExpiryDate, "monthExpiryDate");

    return super.json;
  }
}

/// Bank account detail information
/// -param {String} bankAccountNumber - bank account number
/// -param {String} bankAccountFullname -  bank account owner full name
/// -param {String} bankAccountBsb - bank account state branch
class BankAccountDetails extends ParseObjectToJson {
  String number;
  String fullName;
  String bsb;
  BankAccountDetails({this.number, this.fullName, this.bsb});
  Map<String, dynamic> getJson() {
    parseObject(this.number, "number");
    parseObject(this.fullName, "fullName");
    parseObject(this.bsb, "bsb");

    return super.json;
  }
}

/// Clearing/bank response for transaction
/// -param {String} [verificationStatus]
/// -param {String} [aVSResultCode]
/// -param {String} [declineReasonCode]
/// -param {String} [declineReasonMessage]
/// -param {String} [acquirerResponseCode]
/// -param {String} [acquirerResponseMessage]
/// -param {String} [threeDSResponseStatus]
/// -param {String} [threeDSResponseEnrolled]
/// -param {String} [threeDSResponseECI]
class AuthorizationResponse extends ParseObjectToJson {
  String verificationStatus;
  String aVSResultCode;
  String declineReasonCode;
  String declineReasonMessage;
  String acquirerResponseCode;
  String acquirerResponseMessage;
  String threeDSResponseStatus;
  String threeDSResponseEnrolled;
  String threeDSResponseECI;
  AuthorizationResponse(
      {this.verificationStatus,
      this.aVSResultCode,
      this.declineReasonCode,
      this.declineReasonMessage,
      this.acquirerResponseCode,
      this.acquirerResponseMessage,
      this.threeDSResponseStatus,
      this.threeDSResponseEnrolled,
      this.threeDSResponseECI});
  Map<String, dynamic> getJson() {
    parseObject(this.verificationStatus, "verificationStatus");
    parseObject(this.aVSResultCode, "aVSResultCode");
    parseObject(this.declineReasonCode, "declineReasonCode");
    parseObject(this.declineReasonMessage, "declineReasonMessage");
    parseObject(this.acquirerResponseMessage, "acquirerResponseMessage");
    parseObject(this.threeDSResponseStatus, "threeDSResponseStatus");
    parseObject(this.threeDSResponseEnrolled, "threeDSResponseEnrolled");
    parseObject(this.threeDSResponseECI, "threeDSResponseECI");
    return super.json;
  }
}

/// Types of Checkpoint for risk analysis
enum CheckPointType {
  LAUNCH,
  REGISTER,
  LOGIN,
  TRANSACTION,
  UPDATE_DETAILS,
  ADD_PAYMENT_METHOD,
  UPDATE_USER_INFO,
  PASSWORD_CHANGE,
  VIEW_CARD_DETAILS,
  GENERAL
}

/// Types of screens for Biometrics ScreenListener
enum ScreenListenerType {
  REGISTRATION_FORM,
  LOGIN_FORM,
  MAIN_SCREEN,
  CATALOG_SCREEN,
  SEARCH_SCREEN,
  PRODUCT_SCREEN,
  GENERAL_FORM,
  PINCODE,
  TRANSACTION_DETAILS,
  CONTACTS,
  PAYMENT_SCREEN,
  EMULATOR_CHECK,
  GENERAL,
  ADD_PAYMENT_SCREEN,
  WITHDRAW_SCREEN,
  DEPOSIT_SCREEN
}

///Enum for updating the checkpoint final decision status
enum CheckPointStatus {
  ///checkpoint was approved by the application / application server
  APPROVED,

  /// heckpoint was denied / blocked by the application / application server
  DENIED,

  /// checkpoint was canceled by the user - was not completed
  CANCELLED
}

///Types of verification
enum VerificationType { VERIFIED, NOT_VERIFIED, UNKNOWN }

/// Types of payment method
enum PaymentMethodType { CREDIT_CARD, BANK_ACCOUNT, ECHECK, UNKNOWN }

/// types of transaction
enum TransactionType {
  //payment for goods
  PURCHASE,

  /// money transfer to another payee
  MONEY_TRANSFER,

  /// money deposit to store
  DEPOSIT,

  /// money withdraw from store
  WITHDRAW,

  ///external system money deposit
  EXTERNAL_DEPOSIT,

  /// external server to server money transfer
  EXTERNAL_MONEY_TRANSFER,

  INTERNATIONAL_MONEY_TRANSFER,

  DEBIT_CARD,

  CREDIT_CARD,

  GIFT_CARD,

  SAVING_DEPOSIT,

  SAVING_WITHDRAW,

  ///bill payment transaction type
  BILL_PAYMENT
}

/// ISO 4217 3 letter currency codes
enum CurrencyCode {
  ///Afghan Afghani
  AFN,

  ///Euro
  EUR,

  ///Albanian Lek
  ALL,

  ///Algerian Dinar
  DZD,

  ///US Dollar
  USD,
  AOA,
  XCD,

  ///Argentine Peso
  ARS,

  ///rmenian Dram
  AMD,
  AWG,

  ///Australian Dollar
  AUD,

  ///Azerbaijani Manat
  AZN,
  BSD,

  ///Bahraini Dinar
  BHD,

  ///Bangladeshi Taka
  BDT,
  BBD,
  BYN,

  ///Belize Dollar
  BZD,

  ///CFA Franc BCEAO
  XOF,
  BMD,

  ///Indian Rupee
  INR,
  BTN,

  ///Bolivian Boliviano
  BOB,
  BOV,

  ///Bosnia-Herzegovina Convertible Mark
  BAM,

  ///Botswanan Pula
  BWP,

  ///Norwegian Krone
  NOK,

  ///Brazilian Real
  BRL,

  ///Brunei Dollar
  BND,

  ///Bulgarian Lev
  BGN,

  ///Burundian Franc
  BIF,

  ///Cape Verdean Escudo
  CVE,

  ///Cambodian Riel
  KHR,

  ///CFA Franc BEAC
  XAF,

  ///Canadian Dollar
  CAD,
  KYD,

  ///Chilean Peso
  CLP,
  CLF,

  ///Chinese Yuan
  CNY,

  ///Colombian Peso
  COP,
  COU,

  ///Comorian Franc
  KMF,

  ///Congolese Franc
  CDF,

  ///New Zealand Dollar
  NZD,

  ///Costa Rican Colón
  CRC,

  ///Croatian Kuna
  HRK,
  CUP,
  CUC,
  ANG,

  ///Czech Republic Koruna
  CZK,

  ///Danish Krone
  DKK,

  ///Djiboutian Franc
  DJF,

  ///Dominican Peso
  DOP,

  ///Egyptian Pound
  EGP,
  SVC,

  ///Eritrean Nakfa
  ERN,

  ///Ethiopian Birr
  ETB,
  FKP,
  FJD,
  XPF,
  GMD,

  ///Georgian Lari
  GEL,

  ///Ghanaian Cedi
  GHS,
  GIP,

  ///Guatemalan Quetzal
  GTQ,

  ///British Pound Sterling
  GBP,

  ///Guinean Franc
  GNF,
  GYD,
  HTG,

  ///Honduran Lempira
  HNL,

  ///Hong Kong Dollar
  HKD,

  ///Hungarian Forint
  HUF,

  ///Icelandic Króna
  ISK,

  ///Indonesian Rupiah
  IDR,
  XDR,

  ///Iranian Rial
  IRR,

  ///Iraqi Dinar
  IQD,

  ///Israeli New Sheqel
  ILS,

  ///Jamaican Dollar
  JMD,

  ///Japanese Yen
  JPY,

  ///Jordanian Dinar
  JOD,

  ///Kazakhstani Tenge
  KZT,

  ///Kenyan Shilling
  KES,
  KPW,

  ///South Korean Won
  KRW,

  ///Kuwaiti Dinar
  KWD,
  KGS,
  LAK,

  ///ֿLebanese Pound
  LBP,
  LSL,

  ///South African Rand
  ZAR,
  LRD,

  ///Libyan Dinar
  LYD,

  ///Swiss Franc
  CHF,

  ///Macanese Pataca
  MOP,

  ///Macedonian Denar
  MKD,

  ///Malagasy Ariary
  MGA,
  MWK,

  ///Malaysian Ringgit
  MYR,
  MVR,
  MRU,

  ///Mauritian Rupee
  MUR,
  XUA,

  ///Mexican Peso
  MXN,
  MXV,

  ///Moldovan Leu
  MDL,
  MNT,

  ///Moroccan Dirham
  MAD,

  ///Mozambican Metical
  MZN,

  ///Myanma Kyat
  MMK,

  ///Namibian Dollar
  NAD,

  ///Nepalese Rupee
  NPR,

  ///Nicaraguan Córdoba
  NIO,

  ///Nigerian Naira
  NGN,

  ///Omani Rial
  OMR,

  ///Pakistani Rupee
  PKR,

  ///Panamanian Balboa
  PAB,
  PGK,

  ///Paraguayan Guarani
  PYG,

  ///Peruvian Nuevo Sol
  PEN,

  ///hilippine Peso
  PHP,

  ///Polish Zloty
  PLN,

  /// Qatari Rial
  QAR,

  ///Romanian Leu
  RON,

  ///Russian Ruble
  RUB,

  ///Rwandan Franc
  RWF,
  SHP,
  WST,
  STN,

  ///Saudi Riyal
  SAR,

  ///Serbian Dinar
  RSD,
  SCR,
  SLL,

  ///Singapore Dollar
  SGD,
  XSU,
  SBD,

  ///Somali Shilling
  SOS,
  SSP,

  ///Sri Lankan Rupee
  LKR,

  ///Sudanese Pound
  SDG,
  SRD,
  SZL,

  ///Swedish Krona
  SEK,
  CHE,
  CHW,

  ///Syrian Pound
  SYP,

  ///New Taiwan Dollar
  TWD,
  TJS,

  ///Tanzanian Shilling
  TZS,

  ///Thai Baht
  THB,

  ///Tongan Paʻanga
  TOP,

  ///Trinidad and Tobago Dollar
  TTD,

  ///Tunisian Dinar
  TND,

  ///Turkish Lira
  TRY,
  TMT,

  ///Ugandan Shilling
  UGX,

  ///Ukrainian Hryvnia
  UAH,

  ///United Arab Emirates Dirham
  AED,
  USN,

  ///Uruguayan Peso
  UYU,
  UYI,
  UYW,

  ///Uzbekistan Som
  UZS,
  VUV,
  VES,

  ///Vietnamese Dong
  VND,

  ///Yemeni Rial
  YER,
  ZMW,
  ZWL,
  XBA,
  XBB,
  XBC,
  XBD,
  XTS,
  XXX,
  XAU,
  XPD,
  XPT,
  XAG
}
